@@ -12,29 +12,34 @@ from xml.etree.ElementTree import ElementTree
 from gettext import gettext as _
 
 class LanguagePackageStatus(object):
-    def __init__(self, pkg_template):
+    def __init__(self, languageCode, pkg_template):
+        self.languageCode = languageCode
         self.pkgname_template = pkg_template
         self.available = False
         self.installed = False
         self.doChange = False
 
-
 # the language-support information
 class LanguageInformation(object):
-    def __init__(self):
+    def __init__(self, cache, languageCode=None, language=None):
         #FIXME:
         #needs a new structure:
         #languagePkgList[LANGCODE][tr|fn|in|wa]=[packages available for that language in that category]
         #@property for each category
         #@property for each LANGCODE
-        self.language = None
-        self.languageCode = None
+        self.languageCode = languageCode
+        self.language = language
         # langPack/support status 
         self.languagePkgList = {}
-        self.languagePkgList["languagePack"] = LanguagePackageStatus("language-pack-%s")
-        self.languagePkgList["languageSupportWritingAids"] = LanguagePackageStatus("language-support-writing-%s")
-        self.languagePkgList["languageSupportInputMethods"] = LanguagePackageStatus("language-support-input-%s")
-        self.languagePkgList["languageSupportFonts"] = LanguagePackageStatus("language-support-fonts-%s")
+        self.languagePkgList["languagePack"] = LanguagePackageStatus(languageCode, "language-pack-%s")
+        self.languagePkgList["languageSupportWritingAids"] = LanguagePackageStatus(languageCode, "language-support-writing-%s")
+        self.languagePkgList["languageSupportInputMethods"] = LanguagePackageStatus(languageCode, "language-support-input-%s")
+        self.languagePkgList["languageSupportFonts"] = LanguagePackageStatus(languageCode, "language-support-fonts-%s")
+        for langpkg_status in self.languagePkgList.itervalues():
+            pkgname = langpkg_status.pkgname_template % languageCode
+            langpkg_status.available = pkgname in cache
+            if langpkg_status.available:
+                langpkg_status.installed = cache[pkgname].is_installed
         
     @property
     def inconsistent(self):
@@ -76,7 +81,7 @@ class LanguageSelectorPkgCache(apt.Cache):
 
     def __init__(self, localeinfo, progress):
         apt.Cache.__init__(self, progress)
-        if self._depcache.BrokenCount > 0:
+        if self._depcache.broken_count > 0:
             raise ExceptionPkgCacheBroken()
         self._localeinfo = localeinfo
         # keep the lists 
@@ -88,6 +93,7 @@ class LanguageSelectorPkgCache(apt.Cache):
         self.langpack_locales = {}
         self.pkg_translations = {}
         self.pkg_writing = {}
+        self.multilang = {} # packages which service multiple languages (e.g. openoffice.org-hyphenation)
         filter_list = {}
         blacklist = []
         
@@ -122,9 +128,18 @@ class LanguageSelectorPkgCache(apt.Cache):
             if (c == 'tr' and lc == ''):
                 filter_list[v] = k
             elif (c == 'wa' and lc != ''):
-                if not lc in self.pkg_writing:
-                    self.pkg_writing[lc] = []
-                self.pkg_writing[lc].append(("%s" % k, "%s" % v))
+                if '|' in lc:
+                    if not v in self.multilang:
+                        self.multilang[v] = []
+                    for l in lc.split('|'):
+                        if not l in self.pkg_writing:
+                            self.pkg_writing[l] = []
+                        self.pkg_writing[l].append(("%s" % k, "%s" % v))
+                        self.multilang[v].append(l)
+                else:
+                    if not lc in self.pkg_writing:
+                        self.pkg_writing[lc] = []
+                    self.pkg_writing[lc].append(("%s" % k, "%s" % v))
 
         # get list of all packages available on the system and filter them
         for item in self.keys():
@@ -175,27 +190,37 @@ class LanguageSelectorPkgCache(apt.Cache):
     @property
     def havePackageLists(self):
         " verify that a network package lists exists "
-        for metaindex in self._list.List:
-            for indexfile in metaindex.IndexFiles:
-                if indexfile.ArchiveURI("").startswith("cdrom:"):
+        for metaindex in self._list.list:
+            for indexfile in metaindex.index_files:
+                if indexfile.archive_uri("").startswith("cdrom:"):
                     continue
-                if indexfile.ArchiveURI("").startswith("http://security.ubuntu.com"):
+                if indexfile.archive_uri("").startswith("http://security.ubuntu.com"):
                     continue
-                if indexfile.Exists and indexfile.HasPackages:
+                if indexfile.label != "Debian Package Index":
+                    continue
+                if indexfile.exists and indexfile.has_packages:
                     return True
         return False
 
     def clear(self):
         """ clear the selections """
-        self._depcache.Init()
+        self._depcache.init()
+
+    def verify_no_unexpected_changes(self):
+        (to_inst, to_rm) = self.getChangesList()
+        # FIXME for Arne (20100527): 
+        #   add code that tests if the to_rm list contains
+        #   only stuff that we expect
+        #....
+        return True
         
     def getChangesList(self):
         to_inst = []
         to_rm = []
-        for pkg in self.getChanges():
-            if pkg.markedInstall or pkg.markedUpgrade:
+        for pkg in self.get_changes():
+            if pkg.marked_install or pkg.marked_upgrade:
                 to_inst.append(pkg.name)
-            if pkg.markedDelete:
+            if pkg.marked_delete:
                 to_rm.append(pkg.name)
         return (to_inst,to_rm)
 
@@ -208,87 +233,123 @@ class LanguageSelectorPkgCache(apt.Cache):
                     "language-pack-%s"%languageCode]
         # see what additional pkgs are needed
         #for (pkg, translation) in self.pkg_translations[languageCode]:
-        #    if pkg in self and self[pkg].isInstalled:
+        #    if pkg in self and self[pkg].is_installed:
         #        pkg_list.append(translation)
         return pkg_list
         
     def tryChangeDetails(self, li):
         " change the status of the support details (fonts, input methods) "
         #print li
-        for item in li.languagePkgList.values():
+        # we iterate over items of type LanguagePackageStatus
+        for (key, item) in li.languagePkgList.iteritems():
             if item.doChange:
+                #print "doChange for", li
                 try:
+                    pkgname = item.pkgname_template % li.languageCode
                     if item.installed:
-                        self[item.pkgname_template % li.languageCode].markDelete()
+                        self[pkgname].mark_delete()
                     else:
-                        self[item.pkgname_template % li.languageCode].markInstall()
+                        self[pkgname].mark_install()
+                    # FIXME: this sucks a bit but is better what we
+                    #        had before. ideally this would be part
+                    #        LanguagePackageInformation or somesuch
+                    if key == "languagePack":
+                        self._mark_additional_translation_packages(item)
+                    if key == "languageSupportWritingAids":
+                        self._mark_additional_writing_aids(item)
                 except SystemError:
-                    pass
-        # Check for additional translation packages
-        item = li.languagePkgList["languagePack"]
-        if ((item.installed and not item.doChange) or (item.available and not item.installed and item.doChange)):
-            if li.languageCode in self.pkg_translations:
-                for (pkg, translation) in self.pkg_translations[li.languageCode]:
+                    raise ExceptionPkgCacheBroken()
+
+    def _mark_additional_translation_packages(self, lang_pack_status):
+        if not lang_pack_status.languageCode in self.pkg_translations:
+            return
+        if lang_pack_status.available and not lang_pack_status.installed:
+                for (pkg, translation) in self.pkg_translations[lang_pack_status.languageCode]:
                     if pkg in self and \
-                       self[pkg].isInstalled and \
-                       not self[translation].isInstalled:
-                        self[translation].markInstall()
+                       (self[pkg].is_installed or \
+                       self[pkg].marked_install or \
+                       self[pkg].marked_upgrade) and \
+                       translation in self and \
+                       ((not self[translation].is_installed and \
+                       not self[translation].marked_install and \
+                       not self[translation].marked_upgrade) or \
+                       self[translation].marked_delete):
+                        self[translation].mark_install()
                         #print ("Will pull: %s" % translation)
-        elif ((not item.installed and not item.doChange) or (item.installed and item.doChange)) :
-            if li.languageCode in self.pkg_translations:
-                for (pkg, translation) in self.pkg_translations[li.languageCode]:
-                    if pkg in self and \
-                       self[pkg].isInstalled and \
-                       self[translation].isInstalled:
-                           self[translation].markDelete()
+        elif lang_pack_status.installed:
+                for (pkg, translation) in self.pkg_translations[lang_pack_status.languageCode]:
+                    if translation in self and \
+                       (self[translation].is_installed or \
+                       self[translation].marked_install or \
+                       self[translation].marked_upgrade):
+                           self[translation].mark_delete()
                            #print ("Will remove: %s" % translation)
 
-        # Check for additional writing aids packages
-        item = li.languagePkgList["languageSupportWritingAids"]
-        if ((item.installed and not item.doChange) or (item.available and not item.installed and item.doChange)):
-            if li.languageCode in self.pkg_writing:
-                for (pkg, pull_pkg) in self.pkg_writing[li.languageCode]:
+    def _mark_additional_writing_aids(self, writing_aid_status):
+        if not writing_aid_status.languageCode in self.pkg_writing:
+            return
+        if writing_aid_status.available and not writing_aid_status.installed:
+                for (pkg, pull_pkg) in self.pkg_writing[writing_aid_status.languageCode]:
                     if not pull_pkg in self:
                         continue
                     if '|' in pkg:
                         # multiple dependencies, if one of them is installed, pull the pull_pkg
                         for p in pkg.split('|'):
                             if p in self and \
-                               self[p].isInstalled and \
-                               not self[pull_pkg].isInstalled:
-                                self[pull_pkg].markInstall()
+                               (self[p].is_installed or \
+                               self[p].marked_install or \
+                               self[p].marked_upgrade) and \
+                               ((not self[pull_pkg].is_installed and \
+                               not self[pull_pkg].marked_install and \
+                               not self[pull_pkg].marked_upgrade) or \
+                               self[pull_pkg].marked_delete):
+                                self[pull_pkg].mark_install()
                                 #print ("Will pull: %s" % pull_pkg)
                     else:
                         if pkg in self and \
-                           self[pkg].isInstalled and \
-                           not self[pull_pkg].isInstalled:
-                            self[pull_pkg].markInstall()
+                           (self[pkg].is_installed or \
+                           self[pkg].marked_install or \
+                           self[pkg].marked_upgrade) and \
+                           ((not self[pull_pkg].is_installed and \
+                           not self[pull_pkg].marked_install and \
+                           not self[pull_pkg].marked_upgrade) or \
+                           self[pull_pkg].marked_delete):
+                            self[pull_pkg].mark_install()
                             #print ("Will pull: %s" % pull_pkg)
-        elif ((not item.installed and not item.doChange) or (item.installed and item.doChange)) :
-            if li.languageCode in self.pkg_writing:
-                for (pkg, pull_pkg) in self.pkg_writing[li.languageCode]:
+        elif writing_aid_status.installed and writing_aid_status.doChange:
+                for (pkg, pull_pkg) in self.pkg_writing[writing_aid_status.languageCode]:
                     if not pull_pkg in self:
                         continue
+                    lcount = 0
+                    pcount = 0
+                    if pull_pkg in self.multilang:
+                        # package serves multiple languages.
+                        # check if other languages on this system still need this package.
+                        for l in self.multilang[pull_pkg]:
+                            p = "language-support-writing-%s" % l
+                            if p in self and \
+                               (self[p].is_installed or \
+                               self[p].marked_install or \
+                               self[p].marked_upgrade) and \
+                               not self[p].marked_delete:
+                                lcount = lcount+1
                     if '|' in pkg:
                         # multiple dependencies, if at least one of them is installed, keep the pull_pkg
                         # only remove pull_pkg if none of the dependencies are installed anymore
-                        count = 0
                         for p in pkg.split('|'):
                             if p in self and \
-                               self[p].isInstalled and \
-                               not self[p].markDelete(True) and \
-                               not self[p].markInstall(True):
-                                count = count+1
-                            if count == 0 and self[pull_pkg].isInstalled:
-                                self[pull_pkg].markDelete()
-                                #print ("Will remove: %s" % pull_pkg)
-                    else:
-                        if pkg in self and \
-                           self[pkg].isInstalled and \
-                           self[pull_pkg].isInstalled:
-                            self[pull_pkg].markDelete()
-                            #print ("Will remove: %s" % pull_pkg)
-        return
+                               (self[p].is_installed or \
+                               self[p].marked_install or \
+                               self[p].marked_upgrade) and \
+                               not self[p].marked_delete:
+                                pcount = pcount+1
+                    if pcount == 0  and lcount == 0 and \
+                        (self[pull_pkg].is_installed or \
+                        self[pull_pkg].marked_install or \
+                        self[pull_pkg].marked_upgrade):
+                        self[pull_pkg].mark_delete()
+                        #print ("Will remove: %s" % pull_pkg)
+
 
     def tryInstallLanguage(self, languageCode):
         """ mark the given language for install """
@@ -296,12 +357,7 @@ class LanguageSelectorPkgCache(apt.Cache):
         for name in self._getPkgList(languageCode):
             if name in self:
                 try:
-                    self[name].markInstall()
-                    to_inst.append(name)
-                except SystemError:
-                    pass
-                try:
-                    self[name].markInstall()
+                    self[name].mark_install()
                     to_inst.append(name)
                 except SystemError:
                     pass
@@ -313,7 +369,7 @@ class LanguageSelectorPkgCache(apt.Cache):
             if name in self:
                 try:
                     # purge
-                    self[name].markDelete(True)
+                    self[name].mark_delete(True)
                     to_rm.append(name)
                 except SystemError:
                     pass
@@ -321,19 +377,13 @@ class LanguageSelectorPkgCache(apt.Cache):
     def getLanguageInformation(self):
         """ returns a list with language packs/support packages """
         res = []
-        for (code,lang) in self._localeinfo._lang.items():
+        for (code, lang) in self._localeinfo._lang.items():
             if code == 'zh':
                 continue
-            li = LanguageInformation()
-            li.languageCode = code
-            li.language = lang
-            for langpkg_status in li.languagePkgList.values() :
-                pkgname = langpkg_status.pkgname_template % code
-                langpkg_status.available = pkgname in self
-                if langpkg_status.available:
-                    langpkg_status.installed = self[pkgname].isInstalled
-            if len(filter(lambda s: s.available, li.languagePkgList.values())) > 0:
+            li = LanguageInformation(self, code, lang)
+            if [s for s in li.languagePkgList.itervalues() if s.available]:
                 res.append(li)
+
         return res
 
 
@@ -341,7 +391,7 @@ if __name__ == "__main__":
 
     from LocaleInfo import LocaleInfo
     datadir = "/usr/share/language-selector"
-    li = LocaleInfo("%s/data/languagelist" % datadir)
+    li = LocaleInfo("languagelist", datadir)
 
     lc = LanguageSelectorPkgCache(li,apt.progress.OpProgress())
     print "available language information"
